@@ -9,9 +9,12 @@ import io
 import json
 import pymongo
 import sys
+import uuid
 from asyncio import AbstractEventLoop
 from datetime import datetime
+from pathlib import Path
 from threading import Thread, Lock
+from typing import Dict
 from urllib import parse
 
 import jsonschema
@@ -184,6 +187,66 @@ def start_background_loop(loop: asyncio.AbstractEventLoop) -> None:
     asyncio.set_event_loop(loop)
     loop.run_forever()
 
+def create_certficate_files(config: Dict) -> Dict:
+    path_uuid = uuid.uuid4().hex
+    try:
+        if config.get('tls_ca_file'):
+            path = f"/opt/mongo/{path_uuid}/ca.pem"
+            ca_cert = Path(path)
+            ca_cert.parent.mkdir(exist_ok=True, parents=True)
+            ca_cert.write_text(create_ssl_string(config['tls_ca_file']))
+            config['tls_ca_file'] = path
+            logger.info(f"CA certificate file created at: {path}")
+
+        if config.get('tls_certificate_key_file'):
+            path = f"/opt/mongo/{path_uuid}/client.pem"
+            client_cert = Path(path)
+            client_cert.parent.mkdir(exist_ok=True, parents=True)
+            client_cert.write_text(create_ssl_string(config['tls_certificate_key_file']))
+            config['tls_certificate_key_file'] = path
+            logger.info(f"Client certificate file created at: {path}")
+    except Exception as e:
+        logger.warn(f"Failed to create certificate: /opt/mongo/{path_uuid}/. {e}")
+    return config
+
+def delete_certficate_files(config: Dict) -> None:
+    try:
+        cert = None
+        if config.get('tls_ca_file'):
+            path = config['tls_ca_file']
+            cert = Path(path)
+            config['tls_ca_file'] = cert.read_text()
+            cert.unlink()
+            logger.info(f"CA certificate file deleted from: {path}")
+
+        if config.get('tls_certificate_key_file'):
+            path = config['tls_certificate_key_file']
+            cert = Path(path)
+            config['tls_certificate_key_file'] = cert.read_text()
+            cert.unlink()
+            logger.info(f"Client certificate file deleted from: {path}")
+
+        if cert is not None:
+            cert.parent.rmdir()
+    except Exception as e:
+        logger.warn(f"Failed to delete certificate: {e}")
+
+def create_ssl_string(ssl_string: str) -> str:
+    tls_certificate_key_list = []
+    split_string = ssl_string.split("-----")
+    if len(split_string) < 4:
+        raise Exception("Invalid PEM format for certificate.")
+    for i in range(len(split_string)):
+        if((i % 2) == 1):
+            tls_certificate_key_list.append("-----")
+            tls_certificate_key_list.append(split_string[i])
+            tls_certificate_key_list.append("-----")
+        else:
+            tls_certificate_key_list.append(split_string[i].replace(' ', '\n'))
+    
+    tls_certificate_key_file = ''.join(tls_certificate_key_list)
+    return tls_certificate_key_file
+
 def get_connection_string(config: dict):
     """
     Generates a MongoClientConnectionString based on configuration
@@ -209,6 +272,12 @@ def get_connection_string(config: dict):
 
     if use_ssl:
         connection_query['tls'] = 'true'
+        if config.get('tls_ca_file'):
+            connection_query['tlsCAFile'] = config['tls_ca_file']
+        if config.get('tls_certificate_key_file'):
+            connection_query['tlsCertificateKeyFile'] = config['tls_certificate_key_file']
+            if config.get('tls_certificate_key_file_password'):
+                connection_query['tlsCertificateKeyFilePassword'] = config['tls_certificate_key_file_password']
 
     if direct_connection:
         connection_query['directConnection'] = 'true'
@@ -237,24 +306,30 @@ def main():
     else:
         raise Exception("Required '--config' parameter was not provided")
 
-    connection_string = get_connection_string(config)
-    db_name = config.get("database")
-    target_collection = config.get("target_collection")
-    hard_delete = config.get("hard_delete", False)
+    try:
+        config = create_certficate_files(config)
+        connection_string = get_connection_string(config)
+        db_name = config.get("database")
+        target_collection = config.get("target_collection")
+        hard_delete = config.get("hard_delete", False)
 
-    client = pymongo.MongoClient(connection_string)
-    db = client[db_name]
-    collection = db[target_collection]
-    record_batch = RecordBatch(config)
-    event_loop = setup_batch_task(collection, record_batch)
-    input_messages = io.TextIOWrapper(sys.stdin.buffer, encoding='utf-8')
-    state = persist_messages(collection, input_messages, record_batch, hard_delete)
-    # There can still be records in the `record_batch` which is not processed,
-    # So, we have to force process it one last time before the workflow terminates.
-    try_upsert(collection, record_batch, force=True)
-    emit_state(state)
-    event_loop.stop()
-    logger.info("Exiting normally...")
+        client = pymongo.MongoClient(connection_string)
+        db = client[db_name]
+        collection = db[target_collection]
+        record_batch = RecordBatch(config)
+        event_loop = setup_batch_task(collection, record_batch)
+        input_messages = io.TextIOWrapper(sys.stdin.buffer, encoding='utf-8')
+        state = persist_messages(collection, input_messages, record_batch, hard_delete)
+        # There can still be records in the `record_batch` which is not processed,
+        # So, we have to force process it one last time before the workflow terminates.
+        try_upsert(collection, record_batch, force=True)
+        emit_state(state)
+        event_loop.stop()
+        logger.info("Exiting normally...")
+    except Exception as e:
+        delete_certficate_files(config)
+        raise e
+    delete_certficate_files(config)
 
 
 if __name__ == '__main__':
